# AUTOGENERATED! DO NOT EDIT! File to edit: 02_evaluateFromSearch.ipynb (unless otherwise specified).

__all__ = ['evaluater']

# Cell
import openai, pandas as pd, numpy as np, datetime, json, time, os
from OpenAISurveyWrapper import wrapper

# Cell
class evaluater(wrapper.wrapper):
    """
    wrapper on the API to load in survey definitions
    a format for displaying

    self.myFormat: a function to format a prompt
    """

    labeled = pd.DataFrame()
    raw = pd.DataFrame()
    examples = pd.DataFrame()
    searchTerms = {}

    def addSearchTerm(self, label:str, searchTerm:str):
        """
        k, this is badly named, but I want to be able to evaluate how search does
        """
        try:
            self.searchTerms[label]["searchTerm"] = searchTerm
        except:
            self.searchTerms[label]= {"searchTerm":searchTerm}

    def removeSearchTerm(self, term:str):
        """
        delete search term
        """
        del self.searchTerms[term]

    def loadExamplesFromDataFrame(self, examples:dict):
        """
        takes a 2 column dataframe; first column should be "text", second "label"
        """
        if len(examples.columns)!=2:
            raise Exception("need 2 columns, first should be data second label")
        self.examples = examples
        self.examples.columns=["text", "label"]


    def printSearchTerms(self):
        """
        print off what I'm going to be searching
        """
        for term in self.searchTerms:
            print("{}: {}".format(term, self.searchTerms[term]["searchTerm"]))

    def optimizeSingleSearchCutoff(self, df:pd.DataFrame, term:str, outputDir:str, trainSize=200, testSize=200, sleepTime:float=2):
        """
        run a search against the df texts to get most similar
        the API returns the same similarity scale when the same term is queried multiple times against different docs

        saves the output by the name of the search query to the outputDir
        """

        if df.columns[0] != "text" or df.columns[1] !="label":
            print("pass a df with columns label and text")
            print(df.columns)
            return

        responses = []
        trgts = df[:trainSize + testSize].copy()
        startIdx = 0
        endIdx = 200
        while startIdx < len(trgts):
            #responses = responses + trgts[startIdx:endIdx].text.to_list()

            responses = responses + openai.Engine("davinci").search(documents=\
                        trgts[startIdx:endIdx].text.to_list(), query=term)["data"]
            startIdx += 200
            endIdx += 200
            time.sleep(sleepTime)

        trgts["seach_sim"] = [float(x["score"]) for x in responses]

        cutoff_f1 = pd.DataFrame()
        for i in range(0, 700):
            trgts["search_pos"] = trgts["seach_sim"].apply(lambda x: 1 if x > i else 0)
            cutoff_f1.at["f1", i] = f1_score(trgts.label[:trainSize], trgts.search_pos[:trainSize])

        cutoff = cutoff_f1.T.idxmax()[0]

        cutoff_f1_test = pd.DataFrame()
        for i in range(0, 700):
            trgts["search_pos"] = trgts["seach_sim"].apply(lambda x: 1 if x > i else 0)
            cutoff_f1_test.at["f1", i] = f1_score(trgts.label[trainSize:], trgts.search_pos[trainSize:])

        if outputDir != "":
            os.makedirs(outputDir, exist_ok=True)
            trgts.to_pickle("{}/{}.pkl".format(outputDir, term))

        return cutoff_f1_test[cutoff][0]

    def generateSearchScoresForLabel(self, outFileName="", trainSize=200, testSize=200):
        """
        autogeneratres the search scores
        """

        results = pd.DataFrame()
        for term in self.searchTerms:
            search_term = self.searchTerms[term]["searchTerm"]
            print(search_term)
            myResult = self.optimizeSingleSearchCutoff(self.examples, search_term, outFileName, trainSize=trainSize, testSize=testSize)
            print(myResult)
            results.at[search_term, "testF1"] = myResult

        self.results = results
        return results


# AUTOGENERATED! DO NOT EDIT! File to edit: 02_evaluateFromSearch-Copy1.ipynb (unless otherwise specified).

__all__ = ['evaluater']

# Cell
import openai, pandas as pd, numpy as np, datetime, json, time
from OpenAISurveyWrapper import wrapper

# Cell
class evaluater(wrapper.wrapper):
    """
    wrapper on the API to load in survey definitions
    a format for displaying

    self.myFormat: a function to format a prompt
    """
    labeled = pd.DataFrame()
    raw = pd.DataFrame()
    examples = pd.DataFrame()
    searchTerms = {}
    prompts = {}
    kwargs = {
        "engine":"davinci",
        "temperature":0,
        "max_tokens":10,
        "stop":"\n",
    }

    def addPrompt(self, label:str, definition:str):
        """
        set definition of label to definition
        """
        try:
            self.definitions[label]["definition"] = definition
        except:
            self.definitions[label]= {"definition":definition}

    def removePrompt(self, label:str):
        """
        set definition of label to definition)
        """
        del self.definitions[label]

    def addSearchTerm(self, label:str, searchTerm:str):
        """
        k, this is badly named, but I want to be able to evaluate how search does
        """
        try:
            self.searchTerms[label]["searchTerm"] = searchTerm
        except:
            self.searchTerms[label]= {"searchTerm":searchTerm}

    def removeSearchTerm(self, term:str):
        """
        delete search term
        """
        del self.searchTerms[term]

    def loadExamplesFromDataFrame(self, examples:dict):
        """
        takes a 2 column dataframe; first column should be "text", second "label"
        """
        if len(examples.columns)!=2:
            raise Exception("need 2 columns, first should be data second label")
        self.examples = examples
        self.examples.columns=["data", "label"]

    def generateResponses(self, queries, prompt, maxQuerySize=1500, verbose=False):
        """
        run a prompt against queries.
        prompt should be formated as "str {} str" to be able to use format
        to stick each query in
        """
        results = []
        for query in queries:
            #r = self.query(prompt=prompt.format(query[:maxQuerySize]), verbose=verbose, **self.kwargs)
            if verbose:
                print(r)
            results.append(r)
        return results

    def train_test_split(self, trainSize=0, testSize=0):
        """
        generates train and test set
        train sets with openai api can be small; max of 10 fills up the prompt window generally
        """
        if testSize==0:
            print("give it a train and test")
            return -1

        working = self.examples.copy()

        train = pd.DataFrame()
        test = pd.DataFrame()

        for label in working.label.unique():
            newEntries = working[working.label==label]
            newEntries = newEntries.sample(min(len(newEntries), trainSize))
            train = pd.concat([train, newEntries])
            working.drop(newEntries.index, inplace=True)
        self.train = train
        self.test = test

    def optimizeSearchCutoff(self, df, term, trainSize=200, testSize=200):
        """
        run a search against the df texts to get most similar
        the API returns the same similarity scale when the same term is queried multiple times against different docs
        """



    def useTermPrompt(self, n=20, overwrite=True, inplace=True, fewShot = 0, labels=[], verbose=False):
        """
        for each label in labels, add to the
        """
        if len(labels) == 0:
            labels = [x for x in self.definitions]

        if n > 0:
            targets = self.test.sample(n)
        else:
            targets = self.test.copy()


        all_responses = {}
        for label in labels:
                prompt = self.definitions[label]["prompt"]
                print(prompt)
                responses = self.generateResponses(targets["data"].to_list(), prompt, verbose=verbose)

                with open("{}/{}".format(self.outdir, datetime.datetime.now().strftime("%Y%m%d%H%m%S") + ".json"), "w") as fh:
                    json.dump({"prompt":prompt, "data":{"queries":targets["data"].to_list(), "preds":responses}}, fh, indent=4)

                all_responses[label]=responses

                targets[label] = responses
                self.labeled = targets
        self.labeled = targets





# AUTOGENERATED! DO NOT EDIT! File to edit: 00_wrapper.ipynb (unless otherwise specified).

__all__ = ['wrapper']

# Cell
import openai, pandas, numpy, datetime, json, os

# Cell
class wrapper:
    """
    Wrapper holds the OpenAI API
    """

    autosave=False

    def __init__(self, outdir="queries"):
        """
        build the wrapper, inheriting above
        """
        self.outdir = outdir
        os.makedirs(outdir, exist_ok=True)

    def add_key(self, key:str):
        """
        sets the openai api key
        """
        openai.api_key = key

    def toggleAutoSave(self, status:bool):
        """
        set self.autosave to true or false
        """
        self.autosave=status

    def query(self, verbose=False, **kwargs):
        """
        send kwargs to the API
        """

        r=""

        if verbose:
            print(kwargs)

        r = openai.Completion.create(**kwargs)

        if self.autosave:
            with open("{}/{}".format(self.outdir, datetime.datetime.now().strftime("%Y%m%d%H%m%S")), "w") as fh:
                json.dump({"promt":kwargs["prompt"], "response":r}, fh, indent=4)

        return r



# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"wrapper": "00_wrapper.ipynb",
         "survey": "01_survey.ipynb",
         "evaluater": "02_evaluateFromSearch.ipynb"}

modules = ["wrapper.py",
           "survey.py",
           "evaluate.py",
           "evaluateSearch.py",
           "None.py"]

doc_url = "https://brockmanmatt.github.io/OpenAISurveyWrapper/"

git_url = "https://github.com/brockmanmatt/OpenAISurveyWrapper/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: 01_survey.ipynb (unless otherwise specified).

__all__ = ['survey']

# Cell
import openai, pandas as pd, numpy as np, datetime, json
from OpenAISurveyWrapper import wrapper

# Cell
class survey(wrapper.wrapper):
    """
    wrapper on the API to load in survey definitions
    a format for displaying

    self.myFormat: a function to format a prompt
    """
    labeled = pd.DataFrame()
    raw = pd.DataFrame()
    examples = pd.DataFrame()
    definitions = {}
    myFormat = """A post is about {} if it {} Consider the following post:\npost: "{}"\nq: Was the post about {} (yes or no)?\na:""".format
    kwargs = {
        "engine":"davinci",
        "temperature":0,
        "max_tokens":10,
        "stop":"\n",
    }

    def setFormat(self, func):
        """
        set prompt format to newFormat
        """
        self.myFormat = func

    def formatQuery(self, *args):
        """
        Sets th query format
        The problem with doing it this way is that it's going to be-
        "label, definition, label, it may make more sense to do it another way"
        """
        return (self.myFormat(*args))

    def addDefinition(self, label:str, definition:str):
        """
        set definition of label to definition)
        """
        try:
            self.definitions[label]["definition"] = definition
        except:
            self.definitions[label]= {"definition":definition}

    def addLabelPrompt(self, label:str, prompt:str):
        """
        set definition of label to definition)
        """
        try:
            self.definitions[label]["prompt"] = prompt
        except:
            self.definitions[label] = {"prompt":prompt}


    def removeDefinition(self, label:str):
        """
        set definition of label to definition)
        """
        del self.definitions[label]

    def loadExamplesFromDataFrame(self, examples:dict):
        """
        takes a 2 column dataframe; first column should be "text", second "label"
        """
        if len(examples.columns)!=2:
            raise Exception("need 2 columns, first should be data second label")
        self.examples = examples
        self.examples.columns=["data", "label"]

    def generateResponses(self, queries, prompt, verbose=False):
        """
        run a prompt against queries.
        prompt should be formated as "str {} str" to be able to use format
        to stick each query in
        """

        results = []
        for query in queries:
            r = self.query(prompt=prompt.format(query[:1500]), verbose=verbose, **self.kwargs)["choices"][0]["text"]
            if verbose:
                print(r)
            results.append(r)
        return results

    def tryLabels(self, n=20, overwrite=True, inplace=True, fewShot = 0, labels=[], verbose=False):
        """
        for each label in labels, add to the
        """
        if len(labels) == 0:
            labels = [x for x in self.definitions]

        if n > 0:
            targets = self.examples.sample(n)
        else:
            targets = self.examples.copy()


        all_responses = {}
        for label in labels:
                #so trying to figure out how to generare the prompt on the fly, doesn't work that well
                prompt = self.formatQuery(label, self.definitions[label]["definition"],"{}", label)
                print(prompt)
                responses = self.generateResponses(targets["data"].to_list(), prompt, verbose=verbose)

                with open("{}/{}".format(self.outdir, datetime.datetime.now().strftime("%Y%m%d%H%m%S") + ".json"), "w") as fh:
                    json.dump({"prompt":prompt, "data":{"queries":targets["data"].to_list(), "preds":responses}}, fh, indent=4)

                all_responses[label]=responses

                targets[label] = responses
                self.labeled = targets
        self.labeled = targets

    def useTermPrompt(self, n=20, overwrite=True, inplace=True, fewShot = 0, labels=[], verbose=False):
        """
        for each label in labels, add to the
        """
        if len(labels) == 0:
            labels = [x for x in self.definitions]

        if n > 0:
            targets = self.examples.sample(n)
        else:
            targets = self.examples.copy()


        all_responses = {}
        for label in labels:
                prompt = self.definitions[label]["prompt"]
                print(prompt)
                responses = self.generateResponses(targets["data"].to_list(), prompt, verbose=verbose)

                with open("{}/{}".format(self.outdir, datetime.datetime.now().strftime("%Y%m%d%H%m%S") + ".json"), "w") as fh:
                    json.dump({"prompt":prompt, "data":{"queries":targets["data"].to_list(), "preds":responses}}, fh, indent=4)

                all_responses[label]=responses

                targets[label] = responses
                self.labeled = targets
        self.labeled = targets


